function [K, Labels, Centroids,maxSim, TotalSim,iteration]=SimClustering(data, T, K, upK);

%S-means: Similarity driven clustering. 
%Version: No.1,  July 05, 2007

%input:  data points,  similarity threshold 
% data:  D x N, M is the dimension of point and N is the number of points
% T:    similarity threshold.   0<=T<=1.


%output:  

%K: initial number of clusters
%Labels:  1 x N,  component value in 1..K
%maxSim: 1 x K, the maximum similarity of each point to centroids
%ToalSim: 1 x iteration, the sum(maxSim)/N
%iteration: 1 x 1, number of iterations


%Feel free to use the codes at your own risk. Reference:
% S-means: Similarity driven Clustering and Its application in
% Gravitationa-wave Astronomy Data Mining
%Author: Hansheng Lei (hansheng.lei@utb.edu)
%         Juan R. Iglessias
%         Lappoon Tang
%         Soma Mukherjee
%         Soumya Mohanty
         



%algorithm procedure:


% step 1.  randomly choose  k centroids. Initialize outputs
N=size(data,2);

I=randperm(N);
Centroids=data(:,I(1:K));

Labels=zeros(1,N);
maxSim=zeros(1,N);



change=1;
%step 2. calculate the similarities from each data points to each centroid.
%Assign point to the group with max similarity and similarity > threshold.
%If a point is not assigned to a group, then it is appended to a new
%cluster.

preLabels1=Labels;

TotalSim=0;
iteration=0;
preCentroids=[];
preSimToCenters=[];


while (change==1),
    change=0;

    NewCluster=[];

%if some Centroids have not changed, then not necessary to calulate
% %distance again.
% %make preCentroids same size as Centroids
tmpCent=zeros(size(Centroids));
n1=min(size(preCentroids,1),size(Centroids,1));  %because the size of preCentroids could be bigger than Centroids
n2=min(size(preCentroids,2),size(Centroids,2));
if ~isempty(preCentroids), tmpCent(1:n1,1:n2)=preCentroids(1:n1, 1:n2);end;


%tmp=sum(abs(preCentroids-Centroids));
%I=find(temp~=0);  %those are changed centroids
tmpI=find(sum(tmpCent-Centroids)~=0);  %those are changed centroids
if isempty(tmpI) && ~isempty(preCentroids),  %no centroids move, so can exit
    change=0;
   % break;
end;

tmpSimToCenters=SimilarityToCenters(data, Centroids(:,tmpI));
preCentroids=Centroids;

%merge new Similarity to preSimilarity
SimToCenters=zeros(size(Centroids,2),N);
if ~isempty(preSimToCenters), SimToCenters(1:size(preSimToCenters,1),1:size(preSimToCenters,2))=preSimToCenters;end;
SimToCenters(tmpI',:)=tmpSimToCenters;

%overwrite previous SimToCenters;
preSimToCenters=SimToCenters;
% 

%SimToCenters=SimilarityToCenters(data, Centroids);  %this sentence
%calucaltes %all points to all centroids directly

    %find the max similarity of each point to each centroid
    [maxSim,Labels]=max(SimToCenters,[],1);
    
    %Assign point to the group with max similarity and similarity > threshold.
    %If  point is not assigned to a group, then it will be appended to a new centroid.
    if size(Centroids,2)<upK,
    tmpIndex=find(maxSim<T);
    NewCluster=data(:,tmpIndex);
    Labels(tmpIndex)=size(Centroids,2)+1;
    end;
    
    
    if length(find(Labels-preLabels1~=0))>0 
        change=1;
        preLabels1=Labels;
        iteration=iteration+1;
        TotalSim(iteration)=mean(maxSim);
    else
        change=0;  %exit
    end;

    
%step 3. update each centroid as the mean of each group members.
% if no change, finish, otherwise, go to step 2.

flag=ones(1,size(Centroids,2));
for i=1:size(Centroids,2),
    tmp=find(Labels==i);
    if length(tmp)>0
      %Centroids(:,i)= mean(data(:,tmp),2);
      if Centroids(:,i)~=mean(data(:,tmp),2),
          change=1;
          Centroids(:,i)= mean(data(:,tmp),2);
      end;
    else
      flag(i)=0;
    end;
end;

%remove centroid who has no element
Centroids=Centroids(:, find(flag==1));
if ~isempty(NewCluster),
    %randomly choose one sample for NewCluster as centroid
    I=randperm(size(NewCluster,2));
    NewCentroid=NewCluster(:,I(1:1));
    Centroids=[Centroids,NewCentroid];
    %Centroids=[Centroids,mean(NewCluster,2)];
   elseif  length(find(TotalSim(1:iteration-1)==TotalSim(iteration)))>0   %K does not change and the TotalSim begins to repeat.   
       change=0;  %exit
end;

   
end;             %end while
        
K=size(Centroids,2);

     



